## General Inquirer category annotators.

General_Inquirer_Category_Annotator <-
function(Categories = NULL, Source = NULL, 
         which = c("all", "first"),
         pos = FALSE,
         map = NULL)
{
    which <- match.arg(which)
    
    tab <- General_Inquirer_categories
    if(!is.null(Source))
        tab <- tab[!is.na(match(tab$Source, Source)), ]
    if(!is.null(Categories))
        tab$Categories <-
            lapply(tab$Categories, intersect, Categories)

    ## Simple case: no POS tags.
    if(!pos) {
        tab <- split(tab$Categories, tab$Lemma)
        tab <- if(which == "all")
            lapply(tab, function(e) unique(unlist(e)))
        else
            lapply(tab, `[[`, 1L)

        g <- function(s, a = Annotation()) {
            a <- a[a$type == "word"]
            lemmas <- lapply(a$features, `[[`, "lemma")
            ind <- lengths(lemmas) == 1L
            a <- a[ind]
            ## <FIXME>
            ## Could make this more efficient by using the unique lemmas
            ## for category lookup.
            categories <- tab[unlist(lemmas[ind])]
            ind <- lengths(categories) > 0L
            a <- a[ind]
            a$features <-
                lapply(categories[ind],
                       function(e) list(GI_categories = e))
            ## </FIXME>            
            a
        }

        return(Annotator(g))
    }

    ## Tricky case: POS tags.
    if(is.null(map))
        map <- Penn_Treebank_POS_tags_map
    ## Put every "synset" in the first marker category found in the
    ## (names of the) map, and use NA/empty otherwise.
    nms <- names(map)
    cat <- sapply(tab$OthTags,
                  function(e) nms[match(e, nms, 0L)][1L])
    cat[is.na(cat)] <- ""
    tab <- split(tab$Categories,
                 sprintf("%s/%s", tab$Lemma, cat))
    tab <- if(which == "all")
        lapply(tab, function(e) unique(unlist(e)))
    else
        lapply(tab, `[[`, 1L)
    
    g <- function(s, a = Annotation()) {
        a <- a[a$type == "word"]
        tags <- lapply(a$features, `[[`, "POS")
        lemmas <- lapply(a$features, `[[`, "lemma")
        ind <- ((lengths(tags) == 1L) &
                (lengths(lemmas) == 1L))
        a <- a[ind]
        pos <- match(unlist(tags[ind]), unlist(map))
        cat <- rep.int(names(map), lengths(map))[pos]
        cat[is.na(cat)] <- ""
        ## <FIXME>
        ## Could make this more efficient by using the unique lemma and
        ## marker category combinations for tag category lookup.
        categories <- tab[sprintf("%s/%s", unlist(lemmas[ind]), cat)]
        ind <- lengths(categories) > 0L
        a <- a[ind]
        a$features <-
            lapply(categories[ind],
                   function(e) list(GI_categories = e))
        ## </FIXME>            
        a
    }
    
    Annotator(g)
}

## For now, provide the following simple map:
##   ^N <-> noun  ^V <-> verb
## and everything else to "other".
Penn_Treebank_POS_tags_map <-
    list(NOUN = c("NN", "NNP", "NNPS", "NNS"),
         SUPV = c("VB", "VBD", "VBG", "VBN", "VBP", "VBZ"))
